@@ -3,8 +3,13 @@
  # @ Create Time: 2022-04-04 13:23:14
 '''
 import os
+import random
 import re
+import string
 import subprocess
+from typing import Union
+import requests
+import webbrowser
 
 
 def _heroku_is_installed() -> bool:
@@ -114,59 +119,132 @@ def _create_procfile(root_path: os.PathLike):
     print(f'dash-tools: deploy-heroku: Created Procfile')
 
 
-# def _verify_procfile(root_path: os.PathLike) -> bool:
-#     """
-#     Verifies that the Procfile is correct:
+def _check_heroku_app_name_available(heroku_app_name: str) -> bool:
+    """
+    Check that the app name is available on Heroku
 
-#     Ex. If "... src app:server ..." is in Procfile, check that
-#         'server' hook exists in src/app.py
-
-#     Returns:
-#         True (Procfile is correct)
-#         False (Procfile is incorrect)
-#     """
-#     with open(os.path.join(root_path, 'Procfile'), 'r') as procfile:
-#         procfile_contents = procfile.read()
-#     if 'web: gunicorn' not in procfile_contents:
-#         return False
-#     return True
-#     # TODO
+    Returns:
+        True if available
+        False if not
+    """
+    try:
+        if requests.get(f'https://{heroku_app_name}.herokuapp.com/', headers={'User-Agent': 'Custom'}).status_code == 404:
+            return True
+    except requests.exceptions.ConnectionError:
+        exit('dash-tools: deploy-heroku: Invalid Heroku app name')
+    return False
 
 
-def _prompt_user_choice(message: str) -> bool:
+def _verify_procfile(root_path: os.PathLike) -> dict:
+    """
+    Verifies that the Procfile is correct:
+
+    Ex. If "... --chdir src app:server ..." is in Procfile, check that
+        'server' hook exists in src/app.py
+
+    Returns:
+        {
+            'valid': True (valid) or False (invalid),
+            'dir': Directory of app,
+            'hook': Hook name
+            'module': Module name
+        }
+    """
+    with open(os.path.join(root_path, 'Procfile'), 'r') as procfile:
+        procfile_contents = procfile.read()
+
+    # Look for --chdir somedir
+    chdir_regex = r"--chdir [a-zA-Z]+"
+    try:
+        chdir = re.search(chdir_regex, procfile_contents).group(0)
+        chdir = chdir.replace('--chdir ', '')
+    except (AttributeError, IndexError):
+        chdir = ''
+
+    # Look for module:hook
+    hook_regex = r"[a-zA-Z]+:[a-zA-Z]+"
+    try:
+        hook = re.search(hook_regex, procfile_contents).group(0)
+        hook_module = hook.split(':')[0] + '.py'
+        hook = hook.split(':')[1]
+    except (AttributeError, IndexError):
+        hook = ''
+        return {
+            'valid': False,
+            'dir': chdir,
+            'hook': hook,
+            'module': hook_module
+        }
+
+    modpath = os.path.join(root_path, chdir, hook_module)
+
+    # Check that the module exists
+    if not os.path.exists(modpath):
+        return {
+            'valid': False,
+            'dir': chdir,
+            'hook': hook,
+            'module': hook_module
+        }
+
+    # Check that the hook exists in the module
+    with open(modpath, 'r') as modfile:
+        modfile_contents = modfile.read()
+
+    # Look for the hook "{hook} =" or "{hook}=" with spaces and newlines
+    hook_regex = f"^([\n]+{hook}\s=|[\n]+{hook}=|{hook}=|{hook}\s=)"
+    try:
+        re.search(hook_regex, modfile_contents, re.MULTILINE).group(0)
+        return {
+            'valid': True,
+            'dir': chdir,
+            'hook': hook,
+            'module': hook_module
+        }
+    except (AttributeError, IndexError) as e:
+        return {
+            'valid': False,
+            'dir': chdir,
+            'hook': hook,
+            'module': hook_module
+        }
+
+
+def _prompt_user_choice(message: str, prompt: str = 'Continue? (y/n) > ', does_repeat: bool = True) -> bool:
     """
     Prompt the user to continue or not.
 
+    Args:
+        message: Message to display to the user
+        prompt: Prompt to display to the user
+        does_repeat: If True, the user will be prompted again if they enter an invalid response
+
     Returns:
         True (y/Y)
         False (n/N)
     """
     print(message)
-    response = input('dash-tools: Continue? (y/n) > ')
+    response = input(f'dash-tools: {prompt}')
     if response.lower() == 'y':
         return True
     elif response.lower() == 'n':
         return False
-    else:
+    elif does_repeat:
         return _prompt_user_choice(message)
+    else:
+        return False
 
 
-def _check_required_files(root_path: os.PathLike) -> bool:
+def _check_required_files_exist(root_path: os.PathLike) -> bool:
     """
     Check for Procfile, runtime.txt, requirements.txt
     """
-    print(f'dash-tools: deploy-heroku: Checking for Procfile, runtime.txt, and requirements.txt')
     deploy_should_continue = True
-    # Check for the Procfile
+    # Check if procfile exists
     if not _check_file_exists(root_path, 'Procfile'):
         if _prompt_user_choice(
                 'dash-tools: deploy-heroku: Required file Procfile not found. Create one automatically?'):
-            if _prompt_user_choice(
-                    f'dash-tools: deploy-heroku: Verify that "server = app.server" is declared in src/app.py! See https://devcenter.heroku.com/articles/procfile'):
-                _create_procfile(root_path)
-                # _verify_procfile(root_path)
-            else:
-                deploy_should_continue = False
+            _create_procfile(root_path)
         else:
             deploy_should_continue = False
     # Check for the Runtime file
@@ -186,6 +264,29 @@ def _check_required_files(root_path: os.PathLike) -> bool:
     return deploy_should_continue
 
 
+def _add_changes_and_push_to_heroku(heroku_app_name: str) -> bool:
+    """
+    Add changes to the repository and push to Heroku
+
+    Returns:
+        True (Success)
+        False (Failure)
+    """
+    # Create a commit to push to Heroku
+    print(f'dash-tools: deploy-heroku: Creating commit to push to Heroku')
+    os.system(f'git add .')
+    os.system(
+        f'git commit -m "Deploy to Heroku for app {heroku_app_name} - dash-tools"')
+    # Push to Heroku
+    print(f'dash-tools: deploy-heroku: Pushing to Heroku')
+    try:
+        subprocess.check_output(
+            f'git push heroku HEAD:master', shell=True)
+    except subprocess.CalledProcessError:
+        return False
+    return True
+
+
 def _create_app_on_heroku(app_name: str) -> bool:
     """
     Create the project on Heroku and return the git remote URL
@@ -210,10 +311,81 @@ def _create_app_on_heroku(app_name: str) -> bool:
     return True
 
 
-def deploy_app_to_heroku(project_root_dir: os.PathLike, heroku_app_name: str):
+def _heroku_remote_already_exists() -> bool:
+    """
+    Check if the heroku remote is set
+
+    Returns:
+        True if already set
+        False if remote "heroku" is not set
+    """
+    regex = r'heroku'
+    try:
+        output = subprocess.check_output('git remote', shell=True)
+        heroku_grep = re.search(regex, output.decode('utf-8'))
+        try:
+            heroku_grep.group(0)
+        except AttributeError:
+            return False
+    except subprocess.CalledProcessError:
+        exit('dash-tools: deploy-heroku: Failed to check if heroku remote is set')
+    return True
+
+
+def _remove_heroku_remote():
+    """
+    Remove the heroku remote
+
+    Returns:
+        True if successful
+        False if failed
+    """
+    print('dash-tools: deploy-heroku: Removing heroku remote')
+    try:
+        subprocess.check_output('git remote rm heroku', shell=True)
+    except subprocess.CalledProcessError:
+        exit('dash-tools: deploy-heroku: Failed to remove heroku remote')
+
+
+def _success_message(heroku_app_name: str):
+    """
+    Print a success message
+    """
+    print(f'\n\ndash-tools: deploy-heroku: Successfully deployed to Heroku!')
+    print(
+        f'dash-tools: deploy-heroku: Published to git remote: "heroku". Make changes with the "git push heroku" command.')
+    print(
+        f'dash-tools: deploy-heroku: Management Page: https://dashboard.heroku.com/apps/{heroku_app_name}')
+    print(
+        f'dash-tools: deploy-heroku: Application Page: https://{heroku_app_name}.herokuapp.com/')
+
+    # Prompt user to open the deployed app
+    if input('dash-tools: deploy-heroku: Enter any key to open in browser or q to exit > ') != 'q':
+        webbrowser.open(f'https://{heroku_app_name}.herokuapp.com/')
+
+
+def _get_heroku_app_name():
+    """
+    Create or generate app name if one isn't provided
+    """
+    print('dash-tools: deploy-heroku: No app name provided. Please type a unique name or press enter to generate one automatically.')
+    app_name = input('dash-tools: App Name (Optional) > ')
+    if app_name == '':
+        # Generate a random app name
+        app_name = ''.join(random.choices(
+            string.ascii_lowercase + string.digits, k=12))
+    print(f'dash-tools: deploy-heroku: Using app name "{app_name}"')
+    return app_name
+
+
+def deploy_app_to_heroku(project_root_dir: os.PathLike, heroku_app_name: Union[str, None]):
     """
     Uses the Heroku CLI to deploy the current project
     """
+    print('dash-tools: deploy-heroku: Starting')
+    if not heroku_app_name:
+        heroku_app_name = _get_heroku_app_name()
+
     # Check if heroku CLI is installed
     if not _heroku_is_installed():
         print(f'dash-tools: deploy-heroku: Heroku CLI not installed!')
@@ -233,44 +405,71 @@ def deploy_app_to_heroku(project_root_dir: os.PathLike, heroku_app_name: str):
             f'dash-tools: deploy-heroku: Did you forget to "git init"? See https://git-scm.com/docs/git-init')
         exit('dash-tools: deploy-heroku: Failed')
 
+    # Check if the project already exists on Heroku if name is specified
+    if not _check_heroku_app_name_available(heroku_app_name):
+        print(
+            f'dash-tools: deploy-heroku: App "{heroku_app_name}" already exists on Heroku!')
+        print(
+            'dash-tools: deploy-heroku: Please choose a unique name that isn\'t already taken.')
+        exit('dash-tools: deploy-heroku: Failed')
+
+    # Check that heroku remote is not already set
+    if _heroku_remote_already_exists():
+        print(f'dash-tools: deploy-heroku: Git remote "heroku" is already set!')
+        print('dash-tools: deploy-heroku: Please choose an option below:')
+        print('\t1. Push to the existing heroku remote')
+        print('\t2. Remove the heroku remote and continue')
+        print('\t3. Abort')
+        should_continue = False
+        while not should_continue:
+            response = input('dash-tools: Choice (1, 2, 3) > ')
+            if response == '1':
+                if not _add_changes_and_push_to_heroku(heroku_app_name):
+                    exit('dash-tools: deploy-heroku: Failed to push to heroku')
+                print('dash-tools: deploy-heroku: Changes pushed to heroku remote')
+                exit('dash-tools: deploy-heroku: Success')
+            elif response == '2':
+                _remove_heroku_remote()
+                should_continue = True
+            elif response == '3':
+                exit('dash-tools: deploy-heroku: Aborted')
+            else:
+                should_continue = False
+
     # Check that the project has necessary files
-    if not _check_required_files(project_root_dir):
+    if not _check_required_files_exist(project_root_dir):
         print('dash-tools: deploy-heroku: Procfile, runtime.txt, and requirements.txt are needed for Heroku deployment.')
         exit('dash-tools: deploy-heroku: Aborted')
 
+    # Check procfile is correct
+    procfile = _verify_procfile(project_root_dir)
+    if not procfile['valid']:
+        print(
+            f'dash-tools: deploy-heroku: Procfile is incorrect. Did you include "{procfile["hook"]} = app.server" after instantiating "app = Dash(...)" in {procfile["dir"]}/{procfile["module"]}?')
+        print(
+            'dash-tools: deploy-heroku: See https://devcenter.heroku.com/articles/procfile')
+        exit('dash-tools: deploy-heroku: Failed')
+
     # Log into Heroku
     if not _login_heroku_successful():
-        print(f'dash-tools: deploy-heroku: Failed to log into Heroku!')
+        print(f'dash-tools: deploy-heroku: Heroku login failed.')
         exit('dash-tools: deploy-heroku: Failed')
 
     # Confirm deployment settings and create the project on Heroku if the user confirms
-    if not _prompt_user_choice(f'dash-tools: deploy-heroku: Please confirm creating app "{heroku_app_name}" on Heroku and adding git remote "heroku".'):
+    if not _prompt_user_choice(f'dash-tools: deploy-heroku: Please confirm creating app {heroku_app_name} on Heroku and adding git remote "heroku".'):
         exit('dash-tools: deploy-heroku: Aborted')
 
     # Create the project on Heroku and capture the git remote URL
     if not _create_app_on_heroku(heroku_app_name):
         exit(
-            f'dash-tools: deploy-heroku: Deploying app {heroku_app_name} to Heroku failed. Check that the app name is valid.')
+            f'dash-tools: deploy-heroku: Deploying app {heroku_app_name} to Heroku failed.')
 
     # Add python buildpack
     print(f'dash-tools: deploy-heroku: Adding python buildpack')
     os.system(f'heroku buildpacks:add heroku/python -a {heroku_app_name}')
 
-    # Create a commit to push to Heroku
-    print(f'dash-tools: deploy-heroku: Creating commit to push to Heroku')
-    os.system(f'git add .')
-    os.system(
-        f'git commit -m "Initial deploy to heroku for {heroku_app_name}"')
-
     # Push to Heroku
-    print(f'dash-tools: deploy-heroku: Pushing to Heroku')
-    os.system(f'git push heroku master')
+    _add_changes_and_push_to_heroku(heroku_app_name)
 
-    print(
-        f'dash-tools: deploy-heroku: Published to git remote: "heroku" on branch "master". Push changes to this branch!')
-    print(f'dash-tools: deploy-heroku: Successfully deployed to Heroku!')
-    print(
-        f'dash-tools: deploy-heroku: Management Page https://dashboard.heroku.com/apps/{heroku_app_name}')
-    print(
-        f'dash-tools: deploy-heroku: Deployed to https://{heroku_app_name}.herokuapp.com/')
-    print(f'dash-tools: deploy-heroku: Finished!')
+    # Print success message and exit
+    _success_message(heroku_app_name)
